# -*- coding: utf-8 -*-
"""
Created on Mon Jul 13 13:46:30 2020

@author: mszyc
"""

import os, mne, sys
import os.path as op
import numpy as np
import numpy.fft as fft
import pandas as pd
import matplotlib.pyplot as plt
#from matplotlib.ticker import FormatStrFormatter
import glob
#from mne.baseline import rescale
from mne.time_frequency import tfr_multitaper

# Stefan
#dir_main = op.join(op.expanduser("~"),'Documents','Box Sync', 'ASSR_study2')
# Malina
dir_main = op.join(op.expanduser("~"),'Box Sync', 'ASSR_study1')
# EEG lab
#dir_main = op.join(op.expanduser("~"), 'Documents', 'Malina_Box','Box Sync', 'ASSR_study2') # EEG lab

sys.path.append(dir_main + '\\MNE')

##mne.set_log_level('WARNING')

# Set file path
dir_epo = op.join(dir_main, 'MNE', 'fif', 'epochs_tf')

# Common variables: 
# We have these electrodes:
# ['Nz', 'Fpz', 'Fz', 'FCz', 'Cz', 'CPz', 'Pz', 'Cheek', 'EOG']
# Use Fz and FCz for audio
chanpick = ['Fz', 'FCz']
conds = {'lo': 1, 'hi': 2}
conds = list(conds)

# Parameters for TF analyses following this tutorial:
# https://mne.tools/0.19/auto_examples/time_frequency/plot_time_frequency_simulated.html#sphx-glr-auto-examples-time-frequency-plot-time-frequency-simulated-py 
freqs = np.arange(25.6, 54.4, 0.64) # frequencies for tf
#freqs = np.arange(5, 50, 0.1) # frequencies for tf -- TAKES AGES...
#[25.6  26.24 26.88 27.52 28.16 28.8  29.44 30.08 30.72 31.36 32.   32.64
# 33.28 33.92 34.56 35.2  35.84 36.48 37.12 37.76 38.4  39.04 39.68 40.32
# 40.96 41.6  42.24 42.88 43.52 44.16 44.8  45.44 46.08 46.72 47.36 48.
# 48.64 49.28 49.92 50.56 51.2  51.84 52.48 53.12 53.76 54.4 ]
coef = 0.5
n_cycles = freqs * coef
time_bandwidth = 2.0


# ========================
# Processing of 10s epochs:
# ========================
os.chdir(dir_epo)  # change directory to averages folder
epo_len = 10
files = glob.glob('*audio_10s-epo.fif')
dir_results = op.join(dir_main, 'results', 'time_freq')

D_sel_pow = [] # save only signal and noise channels for 2 Hz
D_sel_amp = [] # save only signal and noise channels for 2 Hz

D_all_pow = [] # save all channels
D_all_amp = [] # save all channels

D_tf_avg = {i:[] for i in conds}
#D_tf_bas = {i:[] for i in conds}

D_fft_pow = {i:[] for i in conds}
#D_fft_pow_bas = {i:[] for i in conds}

D_fft_amp = {i:[] for i in conds}
#D_fft_amp_bas = {i:[] for i in conds}

D_pow_tim = {i:[] for i in conds}
#D_pow_tim_bas = {i:[] for i in conds}

D_amp_tim = {i:[] for i in conds}
#D_amp_tim_bas = {i:[] for i in conds}
fps = []
for f in files:
    # f = files[1]
    print('\n'+'='*50)
    fp = int(f[2:4])
    print('Subject:', fp)
    print('='*50)
    fps += [fp]
    epo_fp = mne.read_epochs(f, preload=True)
    tmpall_pow = {'fp': fp} # save all channels for power signal
    tmpall_amp = {'fp': fp} # save all channels for amplitude signal
    tmpsel_pow = {'fp': fp} # save amps for only signal and noise channels for 2 Hz
    tmpsel_amp = {'fp': fp} # save amps for only signal and noise channels for 2 Hz
    for cond in conds:
        #cond = 'lo'
        
        # Multitaper
        # The time-window length is T = n_cycles / freq.
        # E.g., With freq = 20 Hz and n_cycles = 10, we get time = 0.5 s. 
        # If time_bandwidth = 2., then frequency smoothing is (2 / time) = 4 Hz.
        
        # with n_cycles = 10.*freq, we get time 10 s.
        # If time_bandwidth = 2., then frequency smoothing is (2/time) = 0.2 Hz.

        print('calculating power')
        power = tfr_multitaper(epo_fp[cond], freqs=freqs, n_cycles = n_cycles,
                       time_bandwidth=time_bandwidth, return_itc=False,
                       picks = ['FCz', 'Fz'])
#        power_bas = power.copy().apply_baseline((None, None), mode = 'mean')
        #D_tf[cond].append(power)#.average())
        D_tf_avg[cond].append(power)
#        D_tf_bas[cond].append(power_bas)

        # Select 40.96 Hz
        # Note that the units are in Volts, so we need to convert it to microV
        # This is why we multiply by 1e6 below
        # ==================
        indx_40 = int(np.where(np.round(power.freqs,2) == 40.96)[0])
        pow40_over_time = power.data[:, indx_40]*1e6
        amp40_over_time = np.sqrt(power.data[:, indx_40]*1e6) # change power to amplitude by sqrt
        D_pow_tim[cond].append(np.mean(pow40_over_time, axis = 0))
        D_amp_tim[cond].append(np.mean(amp40_over_time, axis = 0))
        
#        # Select 40.96 Hz Baseline corrected
#        # ==================
#        indx_40 = int(np.where(np.round(power_bas.freqs,2) == 40.96)[0])
#        pow40_over_time_bas = np.mean(power_bas.data, axis = 0)[indx_40]
#        amp40_over_time_bas = np.sqrt(pow40_over_time_bas) 
#        D_pow_tim_bas[cond].append(pow40_over_time_bas)
#        D_amp_tim_bas[cond].append(amp40_over_time_bas)

        # FFT analysis for f = 2Hz of how the 40 Hz power in time
        # use fft from numpy
        # ==================
#        for signal_tf, i in zip([power, power_bas], ['no_bas','bas']):
        signal_tf = power
        Fs = signal_tf.info['sfreq']
        # select frequencies (by defining min and max below)
        fftfreq = fft.fftfreq(len(signal_tf.times), 1/Fs)
        myfreqs = fftfreq[np.logical_and(fftfreq > 0, fftfreq < 4)]
        freqsindx = np.where(np.isin(fftfreq,myfreqs))[0]

        # index signal and noise frequencies
        foi = 2 # Because our visual stim is every 500ms, so 2 Hz
        indx_signal = int(np.where(np.round(myfreqs, 2) == foi)[0])
        # how many neighboring freqs for the SNR analysis?
        nSNRfreqs = 10
        sur_lo = list(range(indx_signal-2-nSNRfreqs, indx_signal-2, 1))
        sur_hi = list(range(indx_signal+3, indx_signal+3+nSNRfreqs, 1))
        indx_noise = sur_lo + sur_hi
        #([0.8, 0.9, 1. , 1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 
        # 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9, 3. , 3.1, 3.2])
        
        # extract amplitude spectrum for GM (ie evokeds)
        # Do the analyses with signal as power --------------------------------
        signal_foi = pow40_over_time
        fftpows = np.abs(fft.fft(signal_foi, axis = 1))
        fftpows = fftpows[:,freqsindx]
        D_fft_pow[cond].append(np.mean(fftpows, axis = 0))
#        if i == 'no_bas':
#            D_fft_pow[cond].append(np.mean(fftamps, axis = 0))
#        if i == 'bas':
#            D_fft_pow_bas[cond].append(np.mean(fftamps, axis = 0))

        for c in range(0,len(chanpick)):
            for i in range(0,len(myfreqs)):
                tmpall_pow['Amp%s_%s_%.2f'%(cond, chanpick[c],myfreqs[i])] = fftpows[c,i]

            # S, N, SNR, and SmN for all channels
        tmpS = fftpows[:,indx_signal]
        tmpN = np.mean(fftpows[:,indx_noise],axis=1)
        tmpSNR = tmpS/tmpN
        tmpS = np.mean(tmpS)
        tmpN = np.mean(tmpN)
        tmpSNR = np.mean(tmpSNR)
        tmpsel_pow['AmpS'+cond] = tmpS
        tmpsel_pow['AmpN'+cond] = tmpN
        tmpsel_pow['AmpSNR'+cond] = tmpSNR
        tmpsel_pow['AmpSmN'+cond] = tmpS - tmpN     
        
        # Do the analyses with signal as amplitude ----------------------------                
        signal_foi = amp40_over_time
        fftamps = np.abs(fft.fft(signal_foi, axis = 1))
        fftamps = fftamps[:,freqsindx]
        D_fft_amp[cond].append(np.mean(fftamps, axis = 0))
        
        for c in range(0,len(chanpick)):
           for i in range(0,len(myfreqs)):
               tmpall_amp['Amp%s_%s_%.2f'%(cond, chanpick[c],myfreqs[i])] = fftamps[c,i]
                
            # S, N, SNR, and SmN for all channels
        tmpS = fftamps[:,indx_signal]
        tmpN = np.mean(fftamps[:,indx_noise],axis=1)
        tmpSNR = tmpS/tmpN
        tmpS = np.mean(tmpS)
        tmpN = np.mean(tmpN)
        tmpSNR = np.mean(tmpSNR)
        tmpsel_amp['AmpS'+cond] = tmpS
        tmpsel_amp['AmpN'+cond] = tmpN
        tmpsel_amp['AmpSNR'+cond] = tmpSNR
        tmpsel_amp['AmpSmN'+cond] = tmpS - tmpN      
          
    D_sel_pow += [tmpsel_pow]
    D_sel_amp += [tmpsel_amp]
    
    D_all_pow += [tmpall_pow]
    D_all_amp += [tmpall_amp]
    
for i in D_fft_pow:
    D_fft_pow[i] = np.row_stack(D_fft_pow[i])  
#for i in D_fft_pow_bas:
#    D_fft_pow_bas[i] = np.row_stack(D_fft_pow_bas[i])      
    
for i in D_pow_tim:
    D_pow_tim[i] = np.row_stack(D_pow_tim[i])  
#for i in D_pow_tim_bas:
#    D_pow_tim_bas[i] = np.row_stack(D_pow_tim_bas[i])  
    
for i in D_amp_tim:
    D_amp_tim[i] = np.row_stack(D_amp_tim[i])  
#for i in D_amp_tim_bas:
#    D_amp_tim_bas[i] = np.row_stack(D_amp_tim_bas[i])  

# Export data
for df, n, which_data in zip([D_sel_pow, D_sel_amp, D_all_pow, D_all_amp],['power','amplitude','power','amplitude'],['signal_noise','signal_noise','all_channels','all_channels']):
    df = pd.DataFrame(df)
    df.to_csv(op.join(dir_results, 'newunits_data_audio_{}_tf_{}.tsv'.format(which_data, n)), sep='\t', index=False)

#np.save(op.join(dir_results, 'D_amp_{}s.npy'.format(epo_len)), D_amp)
#np.save(op.join(dir_results, 'D_amp_bas_{}s.npy'.format(epo_len)), D_amp_bas)
#np.save(op.join(dir_results, 'D_pow_tim_{}s.npy'.format(epo_len)), D_pow_tim)
#np.save(op.join(dir_results, 'D_pow_tim_bas_{}s.npy'.format(epo_len)), D_pow_tim_bas)
#    
    
# Grand average across subjects
power_GA = {'lo': {}, 'hi': {}}
#power_bas_GA = {'lo': {}, 'hi': {}}
for cond in conds:
    power_GA[cond] = mne.grand_average(D_tf_avg[cond])                                    
#    power_bas_GA[cond] = mne.grand_average(D_tf_bas[cond]) 

print('\n'+'='*50)
print('plotting')
print('='*50)

# Plot with 4 subplots
vmin, vmax =  0.0000000002, 0.0000000006
fmin, fmax = 35, 47 
fig, axs = plt.subplots(1, 2, figsize=(10, 7), sharex=True, sharey=True)
colbar = [0, 1]
labels = ['Low','High']
for cond, ax, cb, lab in zip(conds, axs, colbar, labels):
    power_GA[cond].plot([0], baseline = None, axes = ax, 
            vmin = vmin, vmax = vmax, 
            fmin = fmin, fmax = fmax, colorbar = cb)
    ax.set_title(lab)
fig.tight_layout()
fig.suptitle('FCz and Fz'.format(coef, time_bandwidth))
fig.savefig(op.join(dir_results, 'tf spectrogram for {}_no_baselinecor.jpg'.format(epo_len)), format='jpg', dpi = 600)
plt.close(fig=None)

#        fig, axs = plt.subplots(1, 4, figsize=(20, 7), sharex=True, sharey=True)
#        colbar = [0, 0, 0, 1]
#        labels = ['No','Low','High','Very High']
#        for cond, ax, cb, lab in zip(conds, axs, colbar, labels):
#            power_bas_GA[cond].plot([0], baseline = (0, 0.1), axes = ax, 
#                    vmin = vmin, vmax = vmax, 
#                    fmin = fmin, fmax = fmax, colorbar = cb)
#            ax.set_title(lab)
#        fig.tight_layout()
#        fig.suptitle('n_cycles = freq * {}, time_bandwidth = {}'.format(coef, time_bandwidth))
#        fig.savefig(op.join(dir_results, 'tf representation for freq times {} n_cycles and {} time_bandwidth_baselinecor.jpg'.format(coef, time_bandwidth)), format='jpg', dpi = 600)
#        plt.close(fig=None)


####### PLOTS FROM INDIV DATA, POWER
# use D_pow_tim_bas for baseline corrected data

colors = ['b-','g-']
fig, axs = plt.subplots(2, 1, figsize=(14, 9), sharey=False)
for cond, col, lab in zip(conds, colors, labels):
    axs[0].plot(power_GA[cond].times, np.mean(D_pow_tim[cond], axis = 0), col, label=lab)
axs[0].set_title('power over time for 40.96 Hz')
axs[0].legend(frameon=False)
axs[0].set_xlabel('Time (s)')
axs[0].set_ylabel('Power (µV^2)')
xposition = np.arange(0, max(power_GA[cond].times)+0.5, 0.5).tolist()
for xc in xposition:
    axs[0].axvline(x=xc, color='gray', linestyle='--')
 
#axs[1].plot(myfreqs, np.mean(D_fft_pow['no'], axis = 0), 'k-', label='No')
axs[1].plot(myfreqs, np.mean(D_fft_pow['lo'], axis = 0), 'b-', label='Low')
axs[1].plot(myfreqs, np.mean(D_fft_pow['hi'], axis = 0), 'g-', label='High')
#axs[1].plot(myfreqs, np.mean(D_fft_pow['vh'], axis = 0), 'r-', label='Very High')
axs[1].legend(frameon=False)
axs[1].set_xlabel('Frequency (Hz)')
axs[1].set_ylabel('Power (µV^2)')
#axs[1].spines['top'].set_visible(False)
#axs[1].spines['right'].set_visible(False)
#axs[1].yaxis.set_major_formatter(FormatStrFormatter('%.2f'))
fig.tight_layout()
fig.savefig(op.join(dir_results, '40 Hz power over time and spectrum for {}s_ind.jpg'.format(epo_len)), format='jpg', dpi = 600)
plt.close(fig=None)

####### PLOTS FROM INDIV DATA, AMPLITUDE
# use D_pow_tim_bas for baseline corrected data

colors = ['b-','g-']
fig, axs = plt.subplots(2, 1, figsize=(14, 9), sharey=False)
for cond, col, lab in zip(conds, colors, labels):
    axs[0].plot(power_GA[cond].times, np.mean(D_amp_tim[cond], axis = 0), col, label=lab)
axs[0].set_title('amplitude over time for 40.96 Hz')
axs[0].legend(frameon=False)
axs[0].set_xlabel('Time (s)')
axs[0].set_ylabel('Amplitude (µV)')
xposition = np.arange(0, max(power_GA[cond].times), 0.5).tolist()
for xc in xposition:
    axs[0].axvline(x=xc, color='gray', linestyle='--')
 
#axs[1].plot(myfreqs, np.mean(D_fft_amp['no'], axis = 0), 'k-', label='No')
axs[1].plot(myfreqs, np.mean(D_fft_amp['lo'], axis = 0), 'b-', label='Low')
axs[1].plot(myfreqs, np.mean(D_fft_amp['hi'], axis = 0), 'g-', label='High')
#axs[1].plot(myfreqs, np.mean(D_fft_amp['vh'], axis = 0), 'r-', label='Very High')
axs[1].legend(frameon=False)
axs[1].set_xlabel('Frequency (Hz)')
axs[1].set_ylabel('Amplitude (µV)')
#axs[1].spines['top'].set_visible(False)
#axs[1].spines['right'].set_visible(False)
#axs[1].yaxis.set_major_formatter(FormatStrFormatter('%.2f'))
fig.tight_layout()
fig.savefig(op.join(dir_results, '40 Hz amplitude over time and spectrum for {}s_ind.jpg'.format(epo_len)), format='jpg', dpi = 600)
plt.close(fig=None)



###############################################################################



# ========================
# Processing of 500 ms epochs:
# ========================
os.chdir(dir_epo)  # change directory to averages folder
epo_len = 0.5
files = glob.glob('*audio_500ms-epo.fif')
dir_results = op.join(dir_main, 'results', 'time_freq')

D_tf_avg = {i:[] for i in conds}
#D_tf_bas = {i:[] for i in conds}
fps = []
for f in files:
    # f = files[10]
    print('\n'+'='*50)
    fp = int(f[2:4])
    print('Subject:', fp)
    print('='*50)
    fps += [fp]
    epo_fp = mne.read_epochs(f, preload=True)
    for cond in conds:
        #cond = 'no'
        
        # Multitaper
        # The time-window length is T = n_cycles / freq.
        # E.g., With freq = 20 Hz and n_cycles = 10, we get time = 0.5 s. 
        # If time_bandwidth = 2., then frequency smoothing is (2 / time) = 4 Hz.
        
        # with n_cycles = 10.*freq, we get time 10 s.
        # If time_bandwidth = 2., then frequency smoothing is (2/time) = 0.2 Hz.

        print('calculating power')
        power = tfr_multitaper(epo_fp[cond], freqs=freqs, n_cycles = n_cycles,
                       time_bandwidth=time_bandwidth, return_itc=False,
                       picks = ['FCz', 'Fz'])
#        power_bas = power.copy().apply_baseline((None, None), mode = 'mean')
        #D_tf[cond].append(power)#.average())
        D_tf_avg[cond].append(power)
#        D_tf_bas[cond].append(power_bas)

#        # Select 40.96 Hz
#        # ==================
#        indx_40 = int(np.where(np.round(power.freqs,2) == 40.96)[0])
#        pow40_over_time = np.mean(power.data, axis = 0)[indx_40]
#        D_pow_tim[cond].append(pow40_over_time)

# Grand average across subjects
power_GA = {'no': {}, 'lo': {}, 'hi': {}, 'vh': {}}
#power_bas_GA = {'no': {}, 'lo': {}, 'hi': {}, 'vh': {}}
for cond in conds:
    power_GA[cond] = mne.grand_average(D_tf_avg[cond])                                    
#    power_bas_GA[cond] = mne.grand_average(D_tf_bas[cond]) 


print('\n'+'='*50)
print('plotting')
print('='*50)

# Plot with 4 subplots
vmin, vmax =  0.0000000002, 0.0000000006
fmin, fmax = 35, 47 
fig, axs = plt.subplots(1, 2, figsize=(10, 7), sharex=True, sharey=True)
colbar = [0, 1]
labels = ['Low','High']
for cond, ax, cb, lab in zip(conds, axs, colbar, labels):
    power_GA[cond].plot([0], baseline = None, axes = ax, 
            vmin = vmin, vmax = vmax, 
            fmin = fmin, fmax = fmax, colorbar = cb)
    ax.set_title(lab)
fig.tight_layout()
#fig.suptitle('n_cycles = freq * {}, time_bandwidth = {}'.format(coef, time_bandwidth))
fig.savefig(op.join(dir_results, 'tf spectrogram for {}_no_baselinecor.jpg'.format(epo_len)), format='jpg', dpi = 600)
plt.close(fig=None)



## Below is crap
## =============================================================================
#
####### PLOTS FROM ALREADY AVG DATA 
## Select 40.96 Hz
### ==================
### FFT analysis for f = 2Hz of how the 40 Hz power in time
### use fft from numpy
### ==================
#D_amp_avg = {i:[] for i in conds}
#for cond in conds:
#    #cond = 'no'
#    signal_tf = power_GA[cond]
#    Fs = signal_tf.info['sfreq']
## select frequencies (by defining min and max below)
#    fftfreq = fft.fftfreq(len(signal_tf.times), 1/Fs)
#    myfreqs = fftfreq[np.logical_and(fftfreq > 0, fftfreq < 4)]
#    freqsindx = np.where(np.isin(fftfreq,myfreqs))[0]
#
#    # index signal and noise frequencies
#    foi = 2 # Because our visual stim is every 500ms, so 2 Hz
#    indx_signal = int(np.where(np.round(myfreqs, 2) == foi)[0])
#    # how many neighboring freqs for the SNR analysis?
#    nSNRfreqs = 5
#    sur_lo = list(range(indx_signal-2-nSNRfreqs, indx_signal-2, 1))
#    #for 10: [33.28 33.92 34.56 35.2  35.84 36.48 37.12 37.76 38.4  39.04]
#    sur_hi = list(range(indx_signal+3, indx_signal+3+nSNRfreqs, 1))
#    #for 10: [42.88 43.52 44.16 44.8  45.44 46.08 46.72 47.36 48.   48.64]
#    indx_noise = sur_lo + sur_hi
#
#    # extract amplitude spectrum for GM (ie evokeds)
#    indx_foi = int(np.where(np.round(signal_tf.freqs,2) == 40.96)[0])
#    signal_foi = np.array([[signal_tf.data[0][indx_foi]],[signal_tf.data[1][indx_foi]]])
#    signal_foi = signal_foi.reshape(2,10240)
#    fftamps = np.abs(fft.fft(signal_foi, axis = 1))*1e3
#    fftamps = fftamps[:,freqsindx]
#    D_amp_avg[cond] = np.mean(fftamps, axis = 0)
#
#colors = ['k-','b-','g-','r-']
#indx = int(np.where(np.round(power_GA['no'].freqs,2) == 40.96)[0])
#fig, axs = plt.subplots(2, 1, figsize=(14, 9), sharey=False)
#for cond, col, lab in zip(conds, colors, labels):
#    axs[0].plot(power_GA[cond].times, np.mean(power_GA[cond].data, axis = 0)[indx], col, label=lab)
#axs[0].set_title('power over time for 40.96 Hz')
#axs[0].legend(frameon=False)
#xposition = np.arange(0, max(power_GA[cond].times), 0.5).tolist()
#for xc in xposition:
#    axs[0].axvline(x=xc, color='gray', linestyle='--')
# 
#axs[1].plot(myfreqs, D_amp_avg['no'], 'k-', label='No')
#axs[1].plot(myfreqs, D_amp_avg['lo'], 'b-', label='Low')
#axs[1].plot(myfreqs, D_amp_avg['hi'], 'g-', label='High')
#axs[1].plot(myfreqs, D_amp_avg['vh'], 'r-', label='Very High')
#axs[1].legend(frameon=False)
#axs[1].set_xlabel('Frequencies (Hz)')
#axs[1].set_ylabel('Amplitudes (µV)')
##axs[1].spines['top'].set_visible(False)
##axs[1].spines['right'].set_visible(False)
#axs[1].yaxis.set_major_formatter(FormatStrFormatter('%.2f'))
#fig.tight_layout()
#fig.savefig(op.join(dir_results, '40 Hz power for freq times {} n_cycles and {} time_bandwidth.jpg'.format(coef, time_bandwidth)), format='jpg', dpi = 600)
#plt.close(fig=None)
#
#
#
#
## Baseline corrected
#fig, axs = plt.subplots(figsize=(14, 9), sharey=False)
#for cond, col, lab in zip(conds, colors, labels):
#    plt.plot(power_bas_GA[cond].times, np.mean(power_bas_GA[cond].data, axis = 0)[indx], col, label=lab)
#plt.title('power over time for 40.96 Hz')
#plt.legend(frameon=False)
#xposition = np.arange(0, max(power_GA[cond].times), 0.5).tolist()
#for xc in xposition:
#    plt.axvline(x=xc, color='gray', linestyle='--')
#fig.tight_layout()
#fig.savefig(op.join(dir_results, '40 Hz power for freq times {} n_cycles and {} time_bandwidth_baselinecor.jpg'.format(coef, time_bandwidth)), format='jpg', dpi = 600)
#plt.close(fig=None)
# 
#
#
#
## average over time
#fig, axs = plt.subplots(figsize=(14, 9), sharey=False)
#for cond, col, lab in zip(conds, colors, labels):
#    #plt.xlim(25, 45)
#    #plt.ylim(0, 0.000000001)
#    plt.plot(power_GA[cond].freqs, np.mean(power_GA[cond].data[0], axis= 1), col, label=lab)
#plt.title('spectrum averaged from tf')
#plt.legend(frameon=False)
#fig.tight_layout()
#fig.savefig(op.join(dir_results, 'spectrum for freq times {} n_cycles and {} time_bandwidth.jpg'.format(coef, time_bandwidth)), format='jpg', dpi = 600)
#plt.close(fig=None)
#
##        fig, ax = plt.subplots(figsize=(14, 9), sharey=False)
##        for cond, ax, col, lab in zip(conds, axs, colors, labels):
##            #plt.xlim(25, 45)
##            #plt.ylim(0, 0.000000001)
##            plt.plot(power_bas_GA[cond].freqs, np.mean(power_bas_GA[cond].data[0], axis= 1), col, label=lab)
##        plt.title('spectrum averaged from tf')
##        plt.legend(frameon=False)
##        fig.tight_layout()
##        fig.savefig(op.join(dir_results, 'spectrum for freq times {} n_cycles and {} time_bandwidth_baselinecor.jpg'.format(coef, time_bandwidth)), format='jpg', dpi = 600)
##        plt.close(fig=None)
##        
##################################################################################
## Trying out Hilbert
#
#fps = []
## What plots we get if we average in frequency domain
#D_hilb = {i:[] for i in conds}
#D_hilb_avg = {i:[] for i in conds}
#for f in files:
#    # f = files[10]
#    print('\n'+'='*50)
#    fp = int(f[2:4])
#    print('Subject:', fp)
#    print('='*50)
#    fps += [fp]
#    epo_fp = mne.read_epochs(f, preload=True)
#    epo_fp_hilbert = epo_fp.copy().apply_hilbert(envelope = True)
#    for cond in conds: 
#        D_hilb_avg[cond].append(epo_fp_hilbert[cond].average())    
#        D_hilb[cond].append(epo_fp_hilbert[cond])   
## Grand average across subjects
#hilb_GA = {'no': {}, 'lo': {}, 'hi': {}, 'vh': {}}
#for cond in conds:
#    hilb_GA[cond] = mne.grand_average(D_hilb_avg[cond])
#
#
#frequency_map = list()
#for cond in conds:
#    frequency_map.append(((cond, 40, 42), hilb_GA[cond]))
# 
#from mne.stats import bootstrap_confidence_interval
#
#def stat_fun(x):
#    """Return sum of squares."""
#    return np.sum(x ** 2, axis=0)
#
## Plot
#fig, axes = plt.subplots(4, 1, figsize=(10, 7), sharex=True, sharey=True)
#colors = plt.get_cmap('winter_r')(np.linspace(0, 1, 4))
#for ((cond, fmin, fmax), average), color, ax in zip(
#        frequency_map, colors, axes.ravel()[::-1]):
#    times = average.times * 1e3
#    gfp = np.sum(average.data ** 2, axis=0)
#    gfp = mne.baseline.rescale(gfp, times, baseline=(None))
#    ax.plot(times, gfp, label=cond, color=color, linewidth=2.5)
#    ax.axhline(0, linestyle='--', color='grey', linewidth=2)
#    ci_low, ci_up = bootstrap_confidence_interval(average.data, random_state=0,
#                                                  stat_fun=stat_fun)
#    ci_low = rescale(ci_low, average.times, baseline=(None))
#    ci_up = rescale(ci_up, average.times, baseline=(None))
#    ax.fill_between(times, gfp + ci_up, gfp - ci_low, color=color, alpha=0.3)
#    ax.grid(True)
#    ax.set_ylabel('GFP')
#    ax.annotate('%s (%d-%dHz)' % (cond, fmin, fmax),
#                xy=(0.95, 0.8),
#                horizontalalignment='right',
#                xycoords='axes fraction')
#    ax.set_xlim(0, 10000)
#
#axes.ravel()[-1].set_xlabel('Time [ms]')
##
##
##load = 'no'
##fmin = 40
##fmax = 42
##average = epo_fp['no'].average()
##color = 'red'
##ax = axes.ravel()[::-1][0]
##
##plt.plot(times, gfp, label=load, color=color, linewidth=2.5)
##plt.fill_between(times, gfp + ci_up, gfp - ci_low, color=color, alpha=0.3)
##
#
#
#
## =============================================================================
#
### What plots we get if we average in time domain 
##evo_audio = {i:{} for i in conds}
##D_tf_evo = {i:{} for i in conds}
##for f in files:
##    # f = files[0]
##    print('\n'+'='*50)
##    fp = int(f[2:4])
##    print('Subject:', fp)
##    print('='*50)
##    fps += [fp]
##    epo_fp = mne.read_epochs(f, preload=True)
##    for cond in conds:
##        #cond = 'no'
##        # compute GM (ie evokeds)
##        # =========================
##        ev = epo_fp[cond].average()
##        #ev.set_eeg_reference(ref_channels=['Nz'])
##        ev.info['subject_info'] = fp
##        print(cond + " = " + str(ev.nave) + " trials")
##        evo_audio[cond][fp] = ev
##        chans = [ev.ch_names.index(c) for c in chanpick]
##        
##        power = tfr_multitaper(evo_audio[cond][fp], freqs=freqs, n_cycles=n_cycles,
##                       time_bandwidth=time_bandwidth, return_itc=False, picks = ['FCz', 'Fz'])
##        D_tf_evo[cond][fp] = power
##power.plot(picks = ['FCz', 'Fz'], combine = 'mean')#), vmin=vmin, vmax=vmax,title=cond)
#     
#
#

