import os, mne, sys
import os.path as op
import numpy as np
import numpy.fft as fft
import pandas as pd
import glob

# Stefan
dir_main = op.join(op.expanduser("~"),'Documents','Box Sync', 'ASSR_study1')
# Malina
#dir_main = op.join(op.expanduser("~"),'Box Sync', 'ASSR_study1')

sys.path.append(dir_main + '\\MNE')

mne.set_log_level('WARNING')

# Set file paths
dir_results = op.join(dir_main, 'results')
dir_epo = op.join(dir_main, 'MNE', 'fif', 'epochs')

# data are saved as epochs
# Load filenames
os.chdir(dir_epo)  # change directory to averages folder
files = glob.glob('*audio-minute*.fif')

# ========================
# Process auditory data
# ========================
# This script is for an alternative analysis:
# ====================
# consider development within a block
# compute for each 1-min section with a block
# 100, 200 etc codes for block 1,2,3,4
# 10, 20 codes for load (10=low, 20=high)
# 1, 2, 3 codes for minute within a block
conds = {'b1m1lo': 111, 'b1m2lo': 112,'b1m3lo': 113,
         'b2m1lo': 211, 'b2m2lo': 212,'b2m3lo': 213,
         'b3m1lo': 311, 'b3m2lo': 312,'b3m3lo': 313,
         'b4m1lo': 411, 'b4m2lo': 412,'b4m3lo': 413,
         'b1m1hi': 121, 'b1m2hi': 122,'b1m3hi': 123,
         'b2m1hi': 221, 'b2m2hi': 222,'b2m3hi': 223,
         'b3m1hi': 321, 'b3m2hi': 322,'b3m3hi': 323,
         'b4m1hi': 421, 'b4m2hi': 422,'b4m3hi': 423}
conds = list(conds)

# We have these electrodes:
# ['Nz', 'Fpz', 'Fz', 'FCz', 'Cz', 'M1', 'M2', 'Cheek', 'EOG']
# Use Fz and FCz for audio
chanpick = ['Fz', 'FCz']

# save evoked (i.e., averages)
evo_audio = {i:{} for i in conds}

fps = []
Dall = []
for f in files:
    # f = files[0]
    print('\n')
    print(f)
    fp = int(f[2:4])
    fps += [fp]
    epo_fp = mne.read_epochs(f, preload=True)
    tmpamp = {i:[] for i in conds}
    tmpitc = {i:[] for i in conds}
    for cond in conds:
        #cond = 'b1m1lo'
        # compute GM (ie evokeds)
        # =========================
        ev = epo_fp[cond].average()
        #ev.set_eeg_reference(ref_channels=['Nz'])
        ev.info['subject_info'] = fp
        print(cond + " = " + str(ev.nave) + " trials")
        evo_audio[cond][fp] = ev
        chans = [ev.ch_names.index(c) for c in chanpick]
        
        # use fft from numpy
        # ==================
        # select frequencies (by defining min and max below)
        Fs = ev.info['sfreq']
        fftfreq = fft.fftfreq(len(ev.times), 1/Fs)
        myfreqs = fftfreq[np.logical_and(fftfreq > 25, fftfreq < 55)]
        #[25.6  26.24 26.88 27.52 28.16 28.8  29.44 30.08 30.72 31.36 32.   32.64
        # 33.28 33.92 34.56 35.2  35.84 36.48 37.12 37.76 38.4  39.04 39.68 40.32
        # 40.96 41.6  42.24 42.88 43.52 44.16 44.8  45.44 46.08 46.72 47.36 48.
        # 48.64 49.28 49.92 50.56 51.2  51.84 52.48 53.12 53.76 54.4 ]
        freqsindx = np.where(np.isin(fftfreq,myfreqs))[0]
        
        # index signal and noise frequencies
        ASSRfreq = 40.96
        indx_signal = int(np.where(myfreqs == ASSRfreq)[0])
        indx_noise = []
        # use 10 neighboring freqs for the SNR analysis
        sur_lo = list(range(indx_signal-2-10, indx_signal-2, 1))
        #for 10: [33.28 33.92 34.56 35.2  35.84 36.48 37.12 37.76 38.4  39.04]
        sur_hi = list(range(indx_signal+3, indx_signal+3+10, 1))
        #for 10: [42.88 43.52 44.16 44.8  45.44 46.08 46.72 47.36 48.   48.64]
        indx_noise = sur_lo + sur_hi

        # extract amplitude spectrum for GM (ie evokeds)
        fftamps = np.abs(fft.fft(ev.data[chans], axis = 1))*1e3
        fftamps = fftamps[:,freqsindx]

        tmpS = fftamps[:,indx_signal]
        tmpN = np.mean(fftamps[:,indx_noise],axis=1)
        tmpSNR = tmpS/tmpN
        # mean across channels
        tmpS = np.mean(tmpS)
        tmpN = np.mean(tmpN)
        tmpSNR = np.mean(tmpSNR)
        tmpSmN = tmpS - tmpN
        tmpamp[cond].append([tmpS, tmpN, tmpSNR, tmpSmN])

        # extract itc from individual trials (ie epocheds)
        # =================================================
        # fft results for selected channels
        F = fft.fft(epo_fp[cond].get_data()[:,chans,:], axis=2)
        # the number of epochs
        N = len(F)
        # some computations ...
        fftitc = F/np.abs(F)
        fftitc = np.sum(fftitc, 0)
        fftitc = np.abs(fftitc)/N
        
        # extract selected channels and freq of interest
        fftitc = fftitc[:,freqsindx]
        
        tmpS = fftitc[:,indx_signal]
        tmpN = np.mean(fftitc[:,indx_noise],axis=1)
        tmpSNR = tmpS/tmpN
        # mean across channels
        tmpS = np.mean(tmpS)
        tmpN = np.mean(tmpN)
        tmpSNR = np.mean(tmpSNR)
        tmpSmN = tmpS - tmpN
        tmpitc[cond].append([tmpS, tmpN, tmpSNR, tmpSmN])
    tmpall = {'fp': fp}
    conds_new = ['m1lo', 'm2lo','m3lo','m1hi','m2hi','m3hi']
    for c in conds_new:
        #c='m1lo'
        tmp = np.mean(np.row_stack([tmpamp[conds[i]] 
                      for i in np.where([c in f for f in conds])[0]]), axis = 0)
        tmpall[c+'ampS'] = tmp[0]
        tmpall[c+'ampN'] = tmp[1]
        tmpall[c+'ampSNR'] = tmp[2]
        tmpall[c+'ampSmN'] = tmp[3]
        tmp = np.mean(np.row_stack([tmpitc[conds[i]] 
                      for i in np.where([c in f for f in conds])[0]]), axis = 0)
        tmpall[c+'itcS'] = tmp[0]
        tmpall[c+'itcN'] = tmp[1]
        tmpall[c+'itcSNR'] = tmp[2]
        tmpall[c+'itcSmN'] = tmp[3]
    Dall += [tmpall]
df = pd.DataFrame(Dall)
df.to_csv(op.join(dir_results, 'data_audio_signal_noise_by_minute.csv'), sep='\t', index=False)
print('Done with ', len(fps), ' subjects:\n',fps)