import os, mne, sys
import os.path as op
import numpy as np
import pandas as pd

# Stefan
dir_main = op.join(op.expanduser("~"),'Documents','Box Sync', 'ASSR_study1')
# Malina
#dir_main = op.join(op.expanduser("~"),'Box Sync', 'ASSR_study1')

sys.path.append(dir_main + '\\MNE')
from rejectvisual import rejectvisual

#mne.set_log_level('WARNING')

os.chdir(dir_main)

def preprocess(fp):
    #fp = 1
    print('\n'+'='*80)
    print('Continue with fp %i' % fp)
    print('='*80+'\n')
    # Input files
    bdffile = op.join(dir_main, 'bdf', 'assrs_fp%02d.bdf' % fp)
    logfile = op.join(dir_main, 'logfiles', 'fp_%02d.txt' % fp)
    #bad_channels_file = op.join(dir_main, 'MNE', 'fif', 'bad_channels', 'fp%02d.txt' % fp)
    # Output files
    #evofile_visual = op.join(dir_main, 'MNE', 'fif', 'evokeds', 'fp%02dvisual-ave.fif' % fp)
    #evofile_audio = op.join(dir_main, 'MNE', 'fif', 'evokeds', 'fp%02daudio-ave.fif' % fp)
    epofile_audio = op.join(dir_main, 'MNE', 'fif', 'epochs', 'fp%02daudio-epo.fif' % fp)
    epofile_audio_minute = op.join(dir_main, 'MNE', 'fif', 'epochs', 'fp%02daudio-minute-epo.fif' % fp)
    bad_trials_file_audio = op.join(dir_main, 'MNE', 'fif', 'bad_trials', 'fp%02daudio.txt' % fp)
    bad_trials_file_visual = op.join(dir_main, 'MNE', 'fif', 'bad_trials', 'fp%02dvisual.txt' % fp)
    
    # read bdf file
    raw = mne.io.read_raw_bdf(bdffile, preload=True)
    
    # Apply 0.1 Hz high pass filter (butterworth 4th two-pass)
    mne.io.Raw.filter(raw, l_freq=0.1, h_freq=None, method='iir', iir_params=None)
    #mne.io.Raw.filter(raw, l_freq=0.1, h_freq=None, method='iir', 
    #                  iir_params = dict(order = 2, ftype = 'butter'))
    # If you run the code above, the warning that shows up in the console says: 
    #IIR filter parameters
    #---------------------
    #Butterworth highpass zero-phase (two-pass forward and reverse) non-causal filter:
    #- Filter order 4 (effective, after forward-backward)
    #- Cutoff at 0.10 Hz: -6.02 dB
    # So although the order specified is 2, it shows up as 4.
    #and if you specify the order to be 4, then it shows up as 8.

    # set nose to reference
    raw.set_eeg_reference(ref_channels=['Nz'])

    # create eog (Fpz anode = +, cheek cathode = -) and keep Fpz and Cheek
    raw = mne.set_bipolar_reference(raw, anode=['Fpz'], cathode=['Cheek'], 
                                    ch_name='EOG', drop_refs=False)
    
    Fs = raw.info['sfreq']
    
    # read status channel
    # ======================
    events = mne.find_events(raw=raw, stim_channel='Status', min_duration=0.001)
    # recode decimal to binary
    # StimTracker output is organized as follows:
    # Lightsensor  1 Trig. 1 / 1 (LSB)
    # Lightsensor 2 Trig. 2 / 2
    # Lightsensor 3 Trig. 3 / 4
    # Lightsensor 4 Trig. 4 / 8
    # Audio, left channel Trig. 5 / 16
    # Audio, right channel Trig. 6 / 32
    # Microphone Trig. 7 / 64
    # Event marker bit 0 Trig. 8 / 128
    # Event marker bit 1 Trig. 9 / 1 (MSB)
    # Event marker bit 2 Trig. 10 / 2
    # TTL input, line 1 Trig. 11 / 4
    # TTL input, line 2 Trig. 12 / 8
    # TTL input, line 3 Trig. 13 / 16
    # TTL input, line 4 Trig. 14 / 32
    # TTL input, line 5 Trig. 15 / 64
    # TTL input, line 6 Trig. 16 / 128
    bincodes=[(bin(x)[2:])[::-1] for x in events[:,2]]
    bincodes = [x + '0'*(16-len(x)) for x in bincodes]
    # bincodes organized from left to right to match StimTracker
    
    # extract TTL input
    ttls = [int(x[10:]) for x in bincodes]
    ttls = np.array(ttls)

    # extract info about block onsets and load
    # ========================================
    events = np.hstack((events, np.zeros((events.shape[0],1), dtype = 'int')))
    conds_visual = [100000, 10000, 1000, 100, 111111]
    event_id_visual = ['LowNonTarget',
                       'LowTarget',
                       'HighNonTarget',
                       'HighTarget',
                       'BlockOnset']
    #each block has 360 trials (20% targets) plus 7 nontargets at the beginning.
    #Count   
    #1180    low load nontargets (288*4 + 7*4)
    # 288    low load targets (72*4)
    #1180    high load nontargets (288*4 + 7*4)
    # 288    high load targets (72*4)
    #   8    block beginnings
    print('\n'+'='*80)
    print('Visual events per condition:')
    print('='*80)
    cnt = []
    for i in range(len(conds_visual)):
        indx = np.where(ttls == conds_visual[i])
        print(event_id_visual[i] + ' = {}'.format(np.size(indx)))
        events[indx,3] = (i+1)*10
        cnt.append(np.size(indx))
        # conditions are coded as 10,20,30,40,50
    if not cnt == [1180, 288, 1180, 288, 8]:
        raise Warning("Number of visual trials is off.")

    # ========================
    # Process auditory data
    # ========================
    
    # auditory onsets
    # ================
    # audio is in column 5 and 6
    audioonL = np.array([int(x[4]) for x in bincodes])
    audioonR = np.array([int(x[5]) for x in bincodes])
    print('\n')
    if np.array_equal(audioonL, audioonR):
        print("Data check: Audio L and R are equal")
        # This occurs in 4, 10, 18
    else:
        #raise Warning("Audio L and R differ from each other")
        print("Warning: Audio L and R are not equal.")
        print("Onsets are defined as both are on.")
    # onset is when both are on
    events = np.column_stack((events,np.logical_and(audioonL == 1, audioonR == 1)))
    
    # block onsets (check for subsequent audio onset)
    onsblock = np.where(events[:,3]==50)[0]
    onsaudio = []
    for i in onsblock:
        tmp=events[i:i+10,4] #exstract audio onsets
        onsaudio.append(np.argmax(tmp==1))
    if max(onsaudio) > 2: 
        raise Warning("Block onsets are not followed by audio (within 2 samples)")
    onsaudio = onsblock + onsaudio
    onsdelay = events[onsaudio,0] - events[onsblock,0]
    print('\n'+'='*80)
    print('Auditory block onsets: Delay between block onset and audio onset.')
    print('='*80)
    print('Min  = {} ms'.format(round(np.min(onsdelay)/Fs*1000,2)))
    print('Max  = {} ms'.format(round(np.max(onsdelay)/Fs*1000,2)))
    print('Mean = {} ms'.format(round(np.mean(onsdelay)/Fs*1000,2)))
    print('='*80)
    del(tmp, onsblock, onsdelay)
    
    # move block onset to audio onset
    onsaudio = events[onsaudio,0]
    
    # chop up 3 min after block onset into shorter epochs
    lenepoch = 1.5625 # epoch length in sec, 3.125/2
    nepoch = int(round(180 / lenepoch,0))
    lenepochtf = int(lenepoch * Fs)
    audioepochons = []
    for i in range(0,len(onsaudio)):
        audioepochons = audioepochons + [onsaudio[i] + x * lenepochtf for x in range(nepoch)]
    audioepochons = np.asarray(audioepochons)
    
    # determine condition for each block (from actual next trials)
    evtmp = events[events[:,3] > 0, 3]
    blockaudio = evtmp[np.asarray(np.where(evtmp==50)) + 1][0]
    blockaudio[np.where(blockaudio < 30)] = 1 # low load
    blockaudio[np.where(blockaudio > 20)] = 2 # high load
    # code load and block
    if all(blockaudio == np.tile([1,2],4)):
        audioorder = 1 #to compare later with logfile
        audioepochblocks = np.repeat([1,5,2,6,3,7,4,8],nepoch)
    elif all(blockaudio == np.tile([2,1],4)):
        audioorder = 2
        audioepochblocks = np.repeat([5,1,6,2,7,3,8,4],nepoch)
    else:
        raise Warning("Load levels do not alternate between blocks")
    events_audio = np.column_stack((audioepochons, np.zeros(len(audioepochons)), audioepochblocks)).astype('int')
    
    # auditory epochs
    # ================
    event_id_audio = {'lo1': 1, 'lo2': 2,'lo3': 3,'lo4': 4,
                      'hi1': 5, 'hi2': 6,'hi3': 7,'hi4': 8}
    keepchannels = ['Nz', 'Fpz', 'Fz', 'FCz', 'Cz', 'M1', 'M2', 'Cheek', 'EOG']
    picks = mne.pick_types(raw.info, eeg=True, meg=False, eog=True,
                           selection=keepchannels)
    epochs_params = dict(events=events_audio, event_id = event_id_audio, tmin=0.000976562, tmax=lenepoch,
                         baseline= (None), picks=picks)
    # baseline = whole epoch to get rid of any overall level differences
    # In fact, if you do not use a high-pass filter att all and there is no baseline,
    # the electrode levels differ a lot from each other! Rereferencing does not help with that.
    #
    # If tmin starts at 0, there are 1601 tfs instead of only 1600
    # because it includes time 0. This messes up the frequency analysis
    epochs_audio = mne.Epochs(raw, **epochs_params, preload=True)

    # alternative analysis:
    # ====================
    # consider development within a block
    # compute for each 1-min section with a block
    # 100, 200 etc codes for block 1,2,3,4
    # 10, 20 codes for load (10=low, 20=high)
    # 1, 2, 3 codes for minute within a block
    events_audio[:,2] = np.add(np.hstack([np.repeat(i * 100,nepoch*2) for i in range(1,5)]),
                               np.repeat(blockaudio, nepoch)*10)
    events_audio[:,2] = np.add(events_audio[:,2], 
                               np.tile(np.hstack((np.repeat(1,39), np.repeat(2,38), 
                                                  np.repeat(3,38))), len(blockaudio)))
    # code name reflects block (b1 to b4), minute (m1 to m3), and load (lo or hi)
    event_id_audio_minute = {'b1m1lo': 111, 'b1m2lo': 112,'b1m3lo': 113,
                             'b2m1lo': 211, 'b2m2lo': 212,'b2m3lo': 213,
                             'b3m1lo': 311, 'b3m2lo': 312,'b3m3lo': 313,
                             'b4m1lo': 411, 'b4m2lo': 412,'b4m3lo': 413,
                             'b1m1hi': 121, 'b1m2hi': 122,'b1m3hi': 123,
                             'b2m1hi': 221, 'b2m2hi': 222,'b2m3hi': 223,
                             'b3m1hi': 321, 'b3m2hi': 322,'b3m3hi': 323,
                             'b4m1hi': 421, 'b4m2hi': 422,'b4m3hi': 423}
    epochs_params = dict(events=events_audio, event_id = event_id_audio_minute, tmin=0.000976562, tmax=lenepoch,
                         baseline= (None), picks=picks)
    epochs_audio_minute = mne.Epochs(raw, **epochs_params, preload=True)

    # ========================
    # Process visual data
    # ========================
    # match visual onsets to photodiode 1
    photodiode = np.array([int(x[0]) for x in bincodes])
    events = np.column_stack((events, photodiode))
    onsphoto = np.where(photodiode==1) 
    onsconds = np.where(np.logical_and(events[:,3] > 0, events[:,3]<50))
    onsdelay = events[onsphoto,0] - events[onsconds,0]
    print('\n'+'='*80)
    print('Visual events: delay between condition trigger and photodiode.')
    print('='*80)
    print('Min  = {} ms'.format(round(np.min(onsdelay)/Fs*1000,2)))
    print('Max  = {} ms'.format(round(np.max(onsdelay)/Fs*1000,2)))
    print('Mean = {} ms'.format(round(np.mean(onsdelay)/Fs*1000,2)))
    print('='*80)
    events_visual = events[onsphoto,0:3][0]
    events_visual[:,2] = events[onsconds,3]
    
    # add block number
    # 367*8 = 2936 trials
    blockindx = np.hstack([np.tile(np.hstack((np.repeat(0,7), np.repeat(i,360))),2) for i in range(1,5)])
    events_visual = np.column_stack((events_visual,blockindx))
    # remove the first seven trials in each block
    events_visual = np.delete(events_visual,np.where(blockindx==0), axis=0) #axis 0 deletes rows
    events_visual[:,2] = np.add(events_visual[:,2],events_visual[:,3])
    events_visual = events_visual[:,0:3]
    del(cnt, onsphoto, onsconds, onsdelay, indx, conds_visual, ttls, photodiode, blockindx)

    # dictionary of visual conditions
    event_id_visual = {'Nonlo1': 11,
                       'Tarlo1': 21,
                       'Nonhi1': 31,
                       'Tarhi1': 41,
                       'Nonlo2': 12,
                       'Tarlo2': 22,
                       'Nonhi2': 32,
                       'Tarhi2': 42,
                       'Nonlo3': 13,
                       'Tarlo3': 23,
                       'Nonhi3': 33,
                       'Tarhi3': 43,
                       'Nonlo4': 14,
                       'Tarlo4': 24,
                       'Nonhi4': 34,
                       'Tarhi4': 44}
    
    evcnt = np.unique(events_visual[:,2],return_counts=True)
    print("Condition codes and their frequency:")
    print(evcnt[0])
    print(evcnt[1])
    
    # visual epochs
    # ==============
    epochs_params = dict(events=events_visual, event_id=event_id_visual, 
                         tmin=-.1, tmax=.5, # -100 to +500 ms are preregistered
                         baseline=(None, 0.0), picks=picks)
    epochs_visual = mne.Epochs(raw, **epochs_params, preload=True)
    
    # read logfile (of visual events)
    # ===============================
    logd = pd.read_csv(logfile, sep = '\t')
    if logd['condition'][0] != audioorder:
        raise Warning('The condition codes from the EEG file do not match the logfile!')
    # delete first 7 standards
    logd = logd[logd['trial_nr'] > 7]
    # select relevant columns
    logd = logd[['block_nr','condition','target']]
    logd = logd.values
    logd = np.column_stack((logd, np.zeros(len(logd)))).astype(int)
    # reconstruct codes
    # condition 1 (low load) -> +10
    logd[logd[:,1] ==1, 3] += 10
    # condition 2 (high load) -> +20
    logd[logd[:,1] ==2, 3] += 30
    # target -> +10
    logd[logd[:,2] ==1, 3] += 10
        
    logd[:,3]=logd[:,3]+np.hstack([np.repeat(i,720) for i in range(1,5)])
    if np.any(logd[:,3]!=events_visual[:,2]):
        raise Warning('The visual codes from the EEG file do not match the logfile!')

    # Reject visual
    def cleandata(file, epochs, timeon, timeoff, info):
        if op.isfile(file):
            if os.stat(file).st_size != 0:
                with open(file, 'r') as f:
                    bad_trials = [list(map(int, data.split(','))) for data in f][0]
            else: bad_trials = []
        else:
        #        inp = 'n'
        #        while inp == 'n':
            bad_trials = rejectvisual(epochs, fp, timeon, timeoff, info)
        #    mne.viz.plot_evoked_topo(epochs.average(), title= 'Fp: %s' % fp,
        #                             color=['black'])
        #            inp = input('\nInput n to reset.\nPress enter to continue: ')
        #            if inp == 'n':
        #                print('Resetting.')
        #            else:
        #                print('Removing %i bad trials\n' % len(bad_trials))
            with open(file, 'w') as f:
                f.write(','.join(str(t) for t in bad_trials))
        epochs.drop(bad_trials)

    # remove noisy trials
    print('Clean audio epochs')
    cleandata(bad_trials_file_audio, epochs_audio, 0, lenepoch, "Audio")
    # because the two audio analyses use the same data, just reject the same trials
    cleandata(bad_trials_file_audio, epochs_audio_minute, 0, lenepoch, "Audio")
    print('Clean visual epochs')
    cleandata(bad_trials_file_visual, epochs_visual, -0.1, 0.5, "Visual")

    # save epoched audio
    print("Save audio epochs:")
    for c in list(event_id_audio):
        print(c + " = " + str(len(epochs_audio[c])) + " trials")
    epochs_audio.save(epofile_audio, overwrite=True)

    print("Save audio epochs:")
    for c in list(event_id_audio_minute):
        print(c + " = " + str(len(epochs_audio_minute[c])) + " trials")
    epochs_audio_minute.save(epofile_audio_minute, overwrite=True)

    # save evoked visual
    conds = list(event_id_visual)
    evos = {}
    print("Save visual evokeds:")
    for cond in conds:
        ev = epochs_visual[cond].average()
        print(cond + " = " + str(ev.nave) + " trials")
        evos[cond] = ev
        ev.save(op.join(dir_main, 'MNE', 'fif', 'evokeds', 
                        'fp%02d_visual_' % fp + cond + '-ave.fif' ))
    #We could not write evokeds because nave was messed up in mne19.
    #It used the nave info from first condition for all conditions.
    #mne.write_evokeds(evofile_visual, [evos[cond] for cond in conds])
    
    print('\n'+'='*80)
    print('Preprocessing done for fp %i' % fp)
    print('='*80)

x = int(input('Start from: '))
fps = np.array(range(x,45)) # last subject is 44
fps = np.delete(fps, np.where(fps==23))
# fp = 23: excluded because of electrode failure
for fp in fps:
    preprocess(fp)
