import os, mne, sys
import os.path as op
import numpy as np

# Stefan
#dir_main = op.join(op.expanduser("~"),'Documents','Box Sync', 'ASSR_study1')
# Malina
dir_main = op.join(op.expanduser("~"),'Box Sync', 'ASSR_study1')

sys.path.append(dir_main + '\\MNE')
from rejectvisual import rejectvisual

#mne.set_log_level('WARNING')

os.chdir(dir_main)
def preprocess(fp):
    #fp = 1
    print('\n'+'='*80)
    print('Continue with fp %i' % fp)
    print('='*80+'\n')
    # Input file
    bdffile = op.join(dir_main, 'bdf', 'assrs_fp%02d.bdf' % fp)    
    # read bdf file
    raw = mne.io.read_raw_bdf(bdffile, preload=True)
    
    # Apply 0.1 Hz high pass filter (butterworth 4th two-pass)
    mne.io.Raw.filter(raw, l_freq=0.1, h_freq=None, method='iir', iir_params=None)
    #mne.io.Raw.filter(raw, l_freq=0.1, h_freq=None, method='iir', 
    #                  iir_params = dict(order = 2, ftype = 'butter'))
    # If you run the code above, the warning that shows up in the console says: 
    #IIR filter parameters
    #---------------------
    #Butterworth highpass zero-phase (two-pass forward and reverse) non-causal filter:
    #- Filter order 4 (effective, after forward-backward)
    #- Cutoff at 0.10 Hz: -6.02 dB
    # So although the order specified is 2, it shows up as 4.
    #and if you specify the order to be 4, then it shows up as 8.

    # set nose to reference
    raw.set_eeg_reference(ref_channels=['Nz'])

    # create eog (Fpz anode = +, cheek cathode = -) and keep Fpz and Cheek
    raw = mne.set_bipolar_reference(raw, anode=['Fpz'], cathode=['Cheek'], 
                                    ch_name='EOG', drop_refs=False)
    
    Fs = raw.info['sfreq']
    
    # read status channel
    # ======================
    events = mne.find_events(raw=raw, stim_channel='Status', min_duration=0.001)
    # recode decimal to binary
    # StimTracker output is organized as follows:
    # Lightsensor  1 Trig. 1 / 1 (LSB)
    # Lightsensor 2 Trig. 2 / 2
    # Lightsensor 3 Trig. 3 / 4
    # Lightsensor 4 Trig. 4 / 8
    # Audio, left channel Trig. 5 / 16
    # Audio, right channel Trig. 6 / 32
    # Microphone Trig. 7 / 64
    # Event marker bit 0 Trig. 8 / 128
    # Event marker bit 1 Trig. 9 / 1 (MSB)
    # Event marker bit 2 Trig. 10 / 2
    # TTL input, line 1 Trig. 11 / 4
    # TTL input, line 2 Trig. 12 / 8
    # TTL input, line 3 Trig. 13 / 16
    # TTL input, line 4 Trig. 14 / 32
    # TTL input, line 5 Trig. 15 / 64
    # TTL input, line 6 Trig. 16 / 128
    bincodes=[(bin(x)[2:])[::-1] for x in events[:,2]]
    bincodes = [x + '0'*(16-len(x)) for x in bincodes]
    # bincodes organized from left to right to match StimTracker
    
    # extract TTL input
    ttls = [int(x[10:]) for x in bincodes]
    ttls = np.array(ttls)

    # extract info about block onsets and load
    # ========================================
    events = np.hstack((events, np.zeros((events.shape[0],1), dtype = 'int')))
    conds_visual = [100000, 10000, 1000, 100, 111111]
    event_id_visual = ['LowNonTarget',
                       'LowTarget',
                       'HighNonTarget',
                       'HighTarget',
                       'BlockOnset']
    #each block has 360 trials (20% targets) plus 7 nontargets at the beginning.
    #Count   
    #1180    low load nontargets (288*4 + 7*4)
    # 288    low load targets (72*4)
    #1180    high load nontargets (288*4 + 7*4)
    # 288    high load targets (72*4)
    #   8    block beginnings
    print('\n'+'='*80)
    print('Visual events per condition:')
    print('='*80)
    cnt = []
    for i in range(len(conds_visual)):
        indx = np.where(ttls == conds_visual[i])
        print(event_id_visual[i] + ' = {}'.format(np.size(indx)))
        events[indx,3] = (i+1)*10
        cnt.append(np.size(indx))
        # conditions are coded as 10,20,30,40,50
    if not cnt == [1180, 288, 1180, 288, 8]:
        raise Warning("Number of visual trials is off.")

    # ========================
    # Process data
    # ========================

    # visual onsets
    # ================
    # match visual onsets to photodiode 1
    # matching is slightly different from study 1 because here,
    # at least one subject had too many photodiode events
    
    # create 10s epochs that start at each photodiode onset 
    # (kind of moving average)    
    

    photodiode = np.array([int(x[0]) for x in bincodes])
    events = np.column_stack((events, photodiode))
    onsphoto = np.where(photodiode==1) 
    onsconds = np.where(np.logical_and(events[:,3] > 0, events[:,3]<50))
    onsdelay = events[onsphoto,0] - events[onsconds,0]
    print('\n'+'='*80)
    print('Visual events: delay between condition trigger and photodiode.')
    print('='*80)
    print('Min  = {} ms'.format(round(np.min(onsdelay)/Fs*1000,2)))
    print('Max  = {} ms'.format(round(np.max(onsdelay)/Fs*1000,2)))
    print('Mean = {} ms'.format(round(np.mean(onsdelay)/Fs*1000,2)))
    print('='*80)
    events_tf = events[onsphoto,0:3][0]
    events_tf[:,2] = events[onsconds,3]
        
    loads = events_tf[:, 2]
#        combine_tarnontar = evtmp[np.asarray(np.where(evtmp==80)) + 1][0]
    loads[np.where(np.logical_and(loads >= 10, loads <= 20))] = 1 # low load
    loads[np.where(np.logical_and(loads >= 30, loads <= 40))] = 2 # high load
    events_tf[:,2] = loads

    event_id_tf = {'lo': 1, 'hi': 2}

    evcnt = np.unique(events_tf[:,2],return_counts=True)
    print("Condition codes and their frequency:")
    print(evcnt[0])
    print(evcnt[1])

    # keep all channels (see raw.ch_names)
    keepchannels = ['Nz', 'Fpz', 'Fz', 'FCz', 'Cz', 'M1', 'M2', 'Cheek', 'EOG']
    picks = mne.pick_types(raw.info, eeg=True, meg=False, eog=True,
                           selection=keepchannels)
    
    # epochs
    # ==============
    for lenepoch, lenname in zip([0.5, 10], ['500ms', '10s']):
        epofile_audio_tf = op.join(dir_main, 'MNE', 'fif', 'epochs_tf', 'fp{}audio_{}-epo.fif'.format(str(fp).zfill(2), lenname))
        bad_trials_file_tf = op.join(dir_main, 'MNE', 'fif', 'bad_trials', 'fp{}audio_{}.txt'.format(str(fp).zfill(2), lenname))
        if lenepoch == 10:
#            # don't iclude 19 last events in each block
#            # these epochs will overlap (one epoch starts every 0.5s)
#            epofile_audio_tf_overlap = op.join(dir_main, 'MNE', 'fif', 'epochs_tf', 'fp{}audio_{}_overlap-epo.fif'.format(str(fp).zfill(2), lenname))
#            bad_trials_file_tf_overlap = op.join(dir_main, 'MNE', 'fif', 'bad_trials', 'fp{}audio_{}_overlap.txt'.format(str(fp).zfill(2), lenname))
#
#            tmp_load = np.hstack(np.tile(np.hstack((np.repeat(1,348), np.repeat(0,19))),16))
#            tmp_load = tmp_load * events_tf[:, 2]
#            events_overlap = events_tf.copy()
#            events_overlap[:,2] = tmp_load
#
#            epochs_params_overlap = dict(events=events_overlap, event_id=event_id_tf,
#                             tmin=0.0009765625, tmax=lenepoch, 
#                             baseline=(None), picks=picks)
#            epochs_tf_overlap = mne.Epochs(raw, **epochs_params_overlap, preload=True)
            
            # these epochs will NOT overlap
            # each block has 367 trials, 10 s epoch fits 20 trials. 367 / 20 = 18.35
            # get rid of first 7 trials in each block and 
            # include only first epoch every 20 epochs
            tmp2_load = np.tile(np.hstack((np.repeat(0,7), np.hstack(np.tile(np.hstack((1, np.repeat(0,19))),18)))), 8)
            tmp2_load = tmp2_load * events_tf[:, 2]
            events_tf[:,2] = tmp2_load
            
        epochs_params = dict(events=events_tf, event_id=event_id_tf,
                         tmin=0.0009765625, tmax=lenepoch, #0.0009765625
                         baseline=(None), picks=picks)
        epochs_tf = mne.Epochs(raw, **epochs_params, preload=True)

        # Reject visual
        def cleandata(file, epochs, timeon, timeoff, info):
            if op.isfile(file):
                if os.stat(file).st_size != 0:
                    with open(file, 'r') as f:
                        bad_trials = [list(map(int, data.split(','))) for data in f][0]
                else: bad_trials = []
            else:
            #        inp = 'n'
            #        while inp == 'n':
                bad_trials = rejectvisual(epochs, fp, timeon, timeoff, info)
            #    mne.viz.plot_evoked_topo(epochs.average(), title= 'Fp: %s' % fp,
            #                             color=['black'])
            #            inp = input('\nInput n to reset.\nPress enter to continue: ')
            #            if inp == 'n':
            #                print('Resetting.')
            #            else:
            #                print('Removing %i bad trials\n' % len(bad_trials))
                with open(file, 'w') as f:
                    f.write(','.join(str(t) for t in bad_trials))
            epochs.drop(bad_trials)
    
        # remove noisy trials
        print('Clean audio {} epochs for tf'.format(lenname))   
        cleandata(bad_trials_file_tf, epochs_tf, 0, lenepoch, "Audio")
                 
        # save epoched audio
        print("Save {} audio epochs:".format(lenname))
        for c in list(event_id_tf):
            print(c + " = " + str(len(epochs_tf[c])) + " trials")
        epochs_tf.save(epofile_audio_tf, overwrite=True)

#        # remove noisy trials for the overlapped epochs and save epochs
#        if lenepoch == 10:
#            print('Clean audio {} overlapping epochs for tf'.format(lenname))   
#            cleandata(bad_trials_file_tf_overlap, epochs_tf_overlap, 0, lenepoch, "Audio")  
#            
#            print("Save {} audio overlapping epochs:".format(lenname))
#            for c in list(event_id_tf):
#                print(c + " = " + str(len(epochs_tf_overlap[c])) + " trials")
#            epochs_tf_overlap.save(epofile_audio_tf_overlap, overwrite=True)
             
    print('\n'+'='*50)
    print('Preprocessing done for subject', fp)
    print('='*50)

x = int(input('Start from: '))
fps = np.array(range(x,45)) # last subject is 44
fps = np.delete(fps, np.where(fps==23))
# fp = 23: excluded because of electrode failure
for fp in fps:
    preprocess(fp)
